@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/env python3
 # -*- coding: UTF-8 -*
 
 # Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
@@ -22,33 +22,34 @@ import warnings
 import psutil
 from psutil import WINDOWS
 from psutil._compat import FileNotFoundError
+from psutil._compat import super
 from psutil.tests import APPVEYOR
-from psutil.tests import get_test_subprocess
+from psutil.tests import GITHUB_ACTIONS
 from psutil.tests import HAS_BATTERY
+from psutil.tests import IS_64BIT
 from psutil.tests import mock
+from psutil.tests import PsutilTestCase
 from psutil.tests import PY3
-from psutil.tests import reap_children
+from psutil.tests import PYPY
 from psutil.tests import retry_on_failure
 from psutil.tests import sh
+from psutil.tests import spawn_testproc
+from psutil.tests import terminate
+from psutil.tests import TOLERANCE_DISK_USAGE
 from psutil.tests import unittest
 
-with warnings.catch_warnings():
-    warnings.simplefilter("ignore")
-    try:
-        import win32api  # requires "pip install pypiwin32"
+
+if WINDOWS and not PYPY:
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        import win32api  # requires "pip install pywin32"
         import win32con
         import win32process
         import wmi  # requires "pip install wmi" / "make setup-dev-env"
-    except ImportError:
-        if os.name == 'nt':
-            raise
 
 
 cext = psutil._psplatform.cext
 
-# are we a 64 bit process
-IS_64_BIT = sys.maxsize > 2**32
-
 
 def wrap_exceptions(fun):
     def wrapper(self, *args, **kwargs):
@@ -64,13 +65,20 @@ def wrap_exceptions(fun):
     return wrapper
 
 
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
+@unittest.skipIf(PYPY, "pywin32 not available on PYPY")
+# https://github.com/giampaolo/psutil/pull/1762#issuecomment-632892692
+@unittest.skipIf(GITHUB_ACTIONS and not PY3, "pywin32 broken on GITHUB + PY2")
+class WindowsTestCase(PsutilTestCase):
+    pass
+
+
 # ===================================================================
 # System APIs
 # ===================================================================
 
 
-@unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestCpuAPIs(unittest.TestCase):
+class TestCpuAPIs(WindowsTestCase):
 
     @unittest.skipIf('NUMBER_OF_PROCESSORS' not in os.environ,
                      'NUMBER_OF_PROCESSORS env var is not available')
@@ -108,8 +116,7 @@ class TestCpuAPIs(unittest.TestCase):
         self.assertEqual(proc.MaxClockSpeed, psutil.cpu_freq().max)
 
 
-@unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestSystemAPIs(unittest.TestCase):
+class TestSystemAPIs(WindowsTestCase):
 
     def test_nic_names(self):
         out = sh('ipconfig /all')
@@ -160,6 +167,8 @@ class TestSystemAPIs(unittest.TestCase):
                         break
                     if 'cdrom' in ps_part.opts:
                         break
+                    if ps_part.mountpoint.startswith('A:'):
+                        break  # floppy
                     try:
                         usage = psutil.disk_usage(ps_part.mountpoint)
                     except FileNotFoundError:
@@ -176,6 +185,7 @@ class TestSystemAPIs(unittest.TestCase):
             else:
                 self.fail("can't find partition %s" % repr(ps_part))
 
+    @retry_on_failure()
     def test_disk_usage(self):
         for disk in psutil.disk_partitions():
             if 'cdrom' in disk.opts:
@@ -183,9 +193,9 @@ class TestSystemAPIs(unittest.TestCase):
             sys_value = win32api.GetDiskFreeSpaceEx(disk.mountpoint)
             psutil_value = psutil.disk_usage(disk.mountpoint)
             self.assertAlmostEqual(sys_value[0], psutil_value.free,
-                                   delta=1024 * 1024)
+                                   delta=TOLERANCE_DISK_USAGE)
             self.assertAlmostEqual(sys_value[1], psutil_value.total,
-                                   delta=1024 * 1024)
+                                   delta=TOLERANCE_DISK_USAGE)
             self.assertEqual(psutil_value.used,
                              psutil_value.total - psutil_value.free)
 
@@ -193,7 +203,8 @@ class TestSystemAPIs(unittest.TestCase):
         sys_value = [
             x + '\\' for x in win32api.GetLogicalDriveStrings().split("\\\x00")
             if x and not x.startswith('A:')]
-        psutil_value = [x.mountpoint for x in psutil.disk_partitions(all=True)]
+        psutil_value = [x.mountpoint for x in psutil.disk_partitions(all=True)
+                        if not x.mountpoint.startswith('A:')]
         self.assertEqual(sys_value, psutil_value)
 
     def test_net_if_stats(self):
@@ -213,7 +224,7 @@ class TestSystemAPIs(unittest.TestCase):
             wmi_btime_str, "%Y%m%d%H%M%S")
         psutil_dt = datetime.datetime.fromtimestamp(psutil.boot_time())
         diff = abs((wmi_btime_dt - psutil_dt).total_seconds())
-        self.assertLessEqual(diff, 1)
+        self.assertLessEqual(diff, 3)
 
     def test_boot_time_fluctuation(self):
         # https://github.com/giampaolo/psutil/issues/1007
@@ -232,8 +243,7 @@ class TestSystemAPIs(unittest.TestCase):
 # ===================================================================
 
 
-@unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestSensorsBattery(unittest.TestCase):
+class TestSensorsBattery(WindowsTestCase):
 
     def test_has_battery(self):
         if win32api.GetPwrCapabilities()['SystemBatteriesPresent']:
@@ -293,16 +303,15 @@ class TestSensorsBattery(unittest.TestCase):
 # ===================================================================
 
 
-@unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestProcess(unittest.TestCase):
+class TestProcess(WindowsTestCase):
 
     @classmethod
     def setUpClass(cls):
-        cls.pid = get_test_subprocess().pid
+        cls.pid = spawn_testproc().pid
 
     @classmethod
     def tearDownClass(cls):
-        reap_children()
+        terminate(cls.pid)
 
     def test_issue_24(self):
         p = psutil.Process(0)
@@ -329,20 +338,6 @@ class TestProcess(unittest.TestCase):
         p = psutil.Process(self.pid)
         self.assertRaises(ValueError, p.send_signal, signal.SIGINT)
 
-    def test_exe_and_name(self):
-        for p in psutil.process_iter():
-            # On Windows name() is never supposed to raise AccessDenied,
-            # see https://github.com/giampaolo/psutil/issues/627
-            try:
-                name = p.name()
-            except psutil.NoSuchProcess:
-                pass
-            else:
-                try:
-                    self.assertEqual(os.path.basename(p.exe()), name)
-                except psutil.Error:
-                    continue
-
     def test_num_handles_increment(self):
         p = psutil.Process(os.getpid())
         before = p.num_handles()
@@ -353,46 +348,10 @@ class TestProcess(unittest.TestCase):
         win32api.CloseHandle(handle)
         self.assertEqual(p.num_handles(), before)
 
-    def test_handles_leak(self):
-        # Call all Process methods and make sure no handles are left
-        # open. This is here mainly to make sure functions using
-        # OpenProcess() always call CloseHandle().
-        def call(p, attr):
-            attr = getattr(p, name, None)
-            if attr is not None and callable(attr):
-                attr()
-            else:
-                attr
-
-        p = psutil.Process(self.pid)
-        failures = []
-        for name in dir(psutil.Process):
-            if name.startswith('_') \
-                    or name in ('terminate', 'kill', 'suspend', 'resume',
-                                'nice', 'send_signal', 'wait', 'children',
-                                'as_dict', 'memory_info_ex'):
-                continue
-            else:
-                try:
-                    call(p, name)
-                    num1 = p.num_handles()
-                    call(p, name)
-                    num2 = p.num_handles()
-                except (psutil.NoSuchProcess, psutil.AccessDenied):
-                    pass
-                else:
-                    if num2 > num1:
-                        fail = \
-                            "failure while processing Process.%s method " \
-                            "(before=%s, after=%s)" % (name, num1, num2)
-                        failures.append(fail)
-        if failures:
-            self.fail('\n' + '\n'.join(failures))
-
     @unittest.skipIf(not sys.version_info >= (2, 7),
                      "CTRL_* signals not supported")
     def test_ctrl_signals(self):
-        p = psutil.Process(get_test_subprocess().pid)
+        p = psutil.Process(self.spawn_testproc().pid)
         p.send_signal(signal.CTRL_C_EVENT)
         p.send_signal(signal.CTRL_BREAK_EVENT)
         p.kill()
@@ -527,24 +486,32 @@ class TestProcess(unittest.TestCase):
                 self.assertRaises(psutil.AccessDenied, p.cwd)
         self.assertGreaterEqual(m.call_count, 5)
 
+    def test_exe(self):
+        # NtQuerySystemInformation succeeds if process is gone. Make sure
+        # it raises NSP for a non existent pid.
+        pid = psutil.pids()[-1] + 99999
+        proc = psutil._psplatform.Process(pid)
+        self.assertRaises(psutil.NoSuchProcess, proc.exe)
 
-@unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestProcessWMI(unittest.TestCase):
+
+class TestProcessWMI(WindowsTestCase):
     """Compare Process API results with WMI."""
 
     @classmethod
     def setUpClass(cls):
-        cls.pid = get_test_subprocess().pid
+        cls.pid = spawn_testproc().pid
 
     @classmethod
     def tearDownClass(cls):
-        reap_children()
+        terminate(cls.pid)
 
     def test_name(self):
         w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
         p = psutil.Process(self.pid)
         self.assertEqual(p.name(), w.Caption)
 
+    # This fail on github because using virtualenv for test environment
+    @unittest.skipIf(GITHUB_ACTIONS, "unreliable path on GITHUB_ACTIONS")
     def test_exe(self):
         w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
         p = psutil.Process(self.pid)
@@ -565,15 +532,15 @@ class TestProcessWMI(unittest.TestCase):
         username = "%s\\%s" % (domain, username)
         self.assertEqual(p.username(), username)
 
+    @retry_on_failure()
     def test_memory_rss(self):
-        time.sleep(0.1)
         w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
         p = psutil.Process(self.pid)
         rss = p.memory_info().rss
         self.assertEqual(rss, int(w.WorkingSetSize))
 
+    @retry_on_failure()
     def test_memory_vms(self):
-        time.sleep(0.1)
         w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
         p = psutil.Process(self.pid)
         vms = p.memory_info().vms
@@ -594,8 +561,11 @@ class TestProcessWMI(unittest.TestCase):
         self.assertEqual(wmic_create, psutil_create)
 
 
+# ---
+
+
 @unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestDualProcessImplementation(unittest.TestCase):
+class TestDualProcessImplementation(PsutilTestCase):
     """
     Certain APIs on Windows have 2 internal implementations, one
     based on documented Windows APIs, another one based
@@ -608,21 +578,11 @@ class TestDualProcessImplementation(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        cls.pid = get_test_subprocess().pid
+        cls.pid = spawn_testproc().pid
 
     @classmethod
     def tearDownClass(cls):
-        reap_children()
-    # ---
-    # same tests as above but mimicks the AccessDenied failure of
-    # the first (fast) method failing with AD.
-
-    def test_name(self):
-        name = psutil.Process(self.pid).name()
-        with mock.patch("psutil._psplatform.cext.proc_exe",
-                        side_effect=psutil.AccessDenied(os.getpid())) as fun:
-            self.assertEqual(psutil.Process(self.pid).name(), name)
-            assert fun.called
+        terminate(cls.pid)
 
     def test_memory_info(self):
         mem_1 = psutil.Process(self.pid).memory_info()
@@ -638,14 +598,14 @@ class TestDualProcessImplementation(unittest.TestCase):
 
     def test_create_time(self):
         ctime = psutil.Process(self.pid).create_time()
-        with mock.patch("psutil._psplatform.cext.proc_create_time",
+        with mock.patch("psutil._psplatform.cext.proc_times",
                         side_effect=OSError(errno.EPERM, "msg")) as fun:
             self.assertEqual(psutil.Process(self.pid).create_time(), ctime)
             assert fun.called
 
     def test_cpu_times(self):
         cpu_times_1 = psutil.Process(self.pid).cpu_times()
-        with mock.patch("psutil._psplatform.cext.proc_cpu_times",
+        with mock.patch("psutil._psplatform.cext.proc_times",
                         side_effect=OSError(errno.EPERM, "msg")) as fun:
             cpu_times_2 = psutil.Process(self.pid).cpu_times()
             assert fun.called
@@ -688,7 +648,7 @@ class TestDualProcessImplementation(unittest.TestCase):
 
 
 @unittest.skipIf(not WINDOWS, "WINDOWS only")
-class RemoteProcessTestCase(unittest.TestCase):
+class RemoteProcessTestCase(PsutilTestCase):
     """Certain functions require calling ReadProcessMemory.
     This trivially works when called on the current process.
     Check that this works on other processes, especially when they
@@ -707,44 +667,41 @@ class RemoteProcessTestCase(unittest.TestCase):
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.STDOUT)
             output, _ = proc.communicate()
-            if output == str(not IS_64_BIT):
+            proc.wait()
+            if output == str(not IS_64BIT):
                 return filename
 
-    @classmethod
-    def setUpClass(cls):
-        other_python = cls.find_other_interpreter()
-
-        if other_python is None:
-            raise unittest.SkipTest(
-                "could not find interpreter with opposite bitness")
-
-        if IS_64_BIT:
-            cls.python64 = sys.executable
-            cls.python32 = other_python
-        else:
-            cls.python64 = other_python
-            cls.python32 = sys.executable
-
     test_args = ["-c", "import sys; sys.stdin.read()"]
 
     def setUp(self):
+        super().setUp()
+
+        other_python = self.find_other_interpreter()
+        if other_python is None:
+            raise unittest.SkipTest(
+                "could not find interpreter with opposite bitness")
+        if IS_64BIT:
+            self.python64 = sys.executable
+            self.python32 = other_python
+        else:
+            self.python64 = other_python
+            self.python32 = sys.executable
+
         env = os.environ.copy()
         env["THINK_OF_A_NUMBER"] = str(os.getpid())
-        self.proc32 = get_test_subprocess([self.python32] + self.test_args,
-                                          env=env,
-                                          stdin=subprocess.PIPE)
-        self.proc64 = get_test_subprocess([self.python64] + self.test_args,
-                                          env=env,
-                                          stdin=subprocess.PIPE)
+        self.proc32 = self.spawn_testproc(
+            [self.python32] + self.test_args,
+            env=env,
+            stdin=subprocess.PIPE)
+        self.proc64 = self.spawn_testproc(
+            [self.python64] + self.test_args,
+            env=env,
+            stdin=subprocess.PIPE)
 
     def tearDown(self):
+        super().tearDown()
         self.proc32.communicate()
         self.proc64.communicate()
-        reap_children()
-
-    @classmethod
-    def tearDownClass(cls):
-        reap_children()
 
     def test_cmdline_32(self):
         p = psutil.Process(self.proc32.pid)
@@ -784,7 +741,7 @@ class RemoteProcessTestCase(unittest.TestCase):
 
 
 @unittest.skipIf(not WINDOWS, "WINDOWS only")
-class TestServices(unittest.TestCase):
+class TestServices(PsutilTestCase):
 
     def test_win_service_iter(self):
         valid_statuses = set([
@@ -879,5 +836,5 @@ class TestServices(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    from psutil.tests.runner import run
-    run(__file__)
+    from psutil.tests.runner import run_from_name
+    run_from_name(__file__)
